/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * sendemail/emailmissedcall application
 * 
 * Copyright (C) 2004, Andy Powell 
 *
 */

#include <asterisk/lock.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/channel_pvt.h>
#include <asterisk/pbx.h>
#include <asterisk/options.h>
#include <asterisk/config.h>
#include <asterisk/say.h>
#include <asterisk/module.h>
#include <asterisk/app.h>
#include <asterisk/manager.h>
#include <asterisk/localtime.h>
#include <asterisk/cli.h>
#include <asterisk/utils.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <dirent.h>


#define EMAIL_CONFIG "email.conf"
#define COMMAND_TIMEOUT 5000

#define SENDMAIL "/usr/sbin/sendmail -t"

#define DEFAULT_FROM "Asterisk"
#define DEFAULT_BODY "You missed a call to ${EM_EXTEN} from ${EM_CALLERID} at ${EM_DATE}\n\nAsterisk"
#define DEFAULT_SUBJECT "Missed Call from ${EM_CALLERID} at ${EM_DATE}"


static char *tdesc = "Email notification";

static char *app_sendemail = "SendEmail";
static char *app_emailmissedcall = "EmailMissedCall";

static char *sendemail_synopsis = "SendEmail(to|[from]|subject|body)";
static char *emailmissedcall_synopsis = "EmailMissedCall(to)";

static char *sendemail_descrip =
"SendEmail(to|[from]|subject|body)\n\n"
"Allows the sending of generic email messages from the dialplan. This can be used for notifications etc\n"
"\nUse \\n for newline characters"
"\n";

static char *emailmissedcall_descrip =
"EmailMissedCall(email-address)\n\n"
"Sends an email to a specified email address telling them that they missed a\ncall. The caller id and time are included in the message\n"
"\n";


static char mailcmd[160];
static char mailfrom[160];
static char mailsubject[80];
static char mailbody[1024];

static int load_config(void);

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;


static int get_date(char *s, int len)
{
        struct tm tm;
        time_t t;
        t = time(0);
        localtime_r(&t,&tm);
        return strftime(s, len, "%a %b %e %r %Z %Y", &tm);
}

static void prep_email_sub_vars(struct ast_channel *chan, char *clid, char *dest, char *date)
{
        /* Prepare variables for substition in email body and subject */
        pbx_builtin_setvar_helper(chan, "EM_EXTEN", dest);
        pbx_builtin_setvar_helper(chan, "EM_CALLERID", (clid ? clid : "an unknown caller"));
        pbx_builtin_setvar_helper(chan, "EM_DATE", date);
}


static int send_em_message(char *eto, char *efrom, char *esubject, char *ebody, char *clid, char *destno)
{
        FILE *p=NULL;
        int pfd;
        char date[256];
        char host[256];
        char who[256];
        char tmp[80] = "/tmp/astmail-XXXXXX";
        char tmp2[256];
        time_t t;
        struct tm tm;
	
        pfd = mkstemp(tmp);

        if (pfd > -1) {
                p = fdopen(pfd, "w");
                if (!p) {
                        close(pfd);
                        pfd = -1;
                }
        }

        if (p) {
                gethostname(host, sizeof(host));
		
                if (strchr(efrom, '@'))
                        strncpy(who, efrom, sizeof(who)-1);
                else {
                        snprintf(who, sizeof(who), "%s@%s", efrom, host);
                }
		
                time(&t);
		ast_localtime(&t,&tm,NULL);
		
                strftime(date, sizeof(date), "%a, %d %b %Y %H:%M:%S %z", &tm);
                fprintf(p, "Date: %s\n", date);
                fprintf(p, "From: %s <%s>\n", efrom, who);
                fprintf(p, "To: %s\n", eto);
				

		if (*esubject) {
			struct ast_channel *tchan = ast_channel_alloc(0);
        	        if (tchan) {
                		char *newsubject;
	                        int mlen = strlen(esubject)*3 + 200;
				if ((newsubject = alloca(mlen))) {
                	        	memset(newsubject, 0, mlen);
                        	        prep_email_sub_vars(tchan,clid,destno,date);
                                	pbx_substitute_variables_helper(tchan,esubject,newsubject,mlen);
					fprintf(p, "Subject: %s\n\n", newsubject);
				}
				else
	                	        ast_log(LOG_WARNING, "Cannot allocate workspace for variable substitution\n");
	                        ast_channel_free(tchan);
			}
			else
				ast_log(LOG_WARNING, "Cannot allocate workspace for variable substitution\n");
			
		}

		if (*ebody) {
			struct ast_channel *tchan = ast_channel_alloc(0);
        	        if (tchan) {

                        char *tmpread, *tmpwrite;

                        /* substitute strings \t and \n into the apropriate characters */
                        tmpread = tmpwrite = ebody;
                        while ((tmpwrite = strchr(tmpread,'\\'))) {
                                int len = strlen("\n");
                                switch (tmpwrite[1]) {
                                        case 'n':
                                                strncpy(tmpwrite+len,tmpwrite+2,strlen(tmpwrite+2)+1);
                                                strncpy(tmpwrite,"\n",len);
                                                break;
                                        case 't':
                                                strncpy(tmpwrite+len,tmpwrite+2,strlen(tmpwrite+2)+1);
                                                strncpy(tmpwrite,"\t",len);
                                                break;
                                        default:
                                                ast_log(LOG_NOTICE, "Substitution routine does not support this character: %c\n",tmpwrite[1]);
                                }
                                tmpread = tmpwrite+len;
                        }

                		char *newbody;
	                        int mlen = strlen(ebody)*3 + 200;
				
				if ((newbody = alloca(mlen))) {
                	        	memset(newbody, 0, mlen);
					prep_email_sub_vars(tchan,clid,destno,date);
                                	pbx_substitute_variables_helper(tchan,ebody,newbody,mlen);
					fprintf(p, "%s\n", newbody);
				}
				else
	                	        ast_log(LOG_WARNING, "Cannot allocate workspace for variable substitution\n");
	                        ast_channel_free(tchan);
			}
			else
				ast_log(LOG_WARNING, "Cannot allocate workspace for variable substitution\n");
		}
        	fclose(p);
		
                snprintf(tmp2, sizeof(tmp2), "( %s < %s ; rm -f %s ) &", mailcmd, tmp, tmp);
                ast_safe_system(tmp2);
                ast_log(LOG_DEBUG, "Sent mail to %s with command '%s'\n", eto, mailcmd);

	}
	else
        	ast_log(LOG_WARNING, "Cannot open a file for write, Hmmmm...\n");
				
        return 0;
}



static int emailmissedcall_exec(struct ast_channel *chan, void *data)
{
	int res=0;
	
        char *emailaddress;
        char *s;
	
        int inum;
	
	struct localuser *u;

	
	if (!data) {
		ast_log(LOG_WARNING, "You didn't pass any arguments - I need an email address\n");
		return -1;
	}
	LOCAL_USER_ADD(u);
	/* She sits alone by a lamp post.... */

        s = ast_strdupa((void *) data);

        emailaddress = strsep(&s, "|");
					
	/* check to see if params passed */

        if (!emailaddress) {
                ast_log(LOG_WARNING, "EmailMissedCall requires an email address as an argument\n");
                LOCAL_USER_REMOVE(u);
                return -1;
	}	
	
	/* Go for it */
	
        res = send_em_message(emailaddress,mailfrom,mailsubject,mailbody,chan->callerid, chan->exten);
	
	LOCAL_USER_REMOVE(u);
	return res;
}



static int sendemail_exec(struct ast_channel *chan, void *data)
{
	int res=0;
	
        char *emailaddress,*temailfrom, *emailsubject, *emailbody;
	char *emailfrom    = DEFAULT_FROM;	
        char *s;
	
        int inum;
	
	struct localuser *u;

	
	if (!data) {
		ast_log(LOG_WARNING, "You didn't pass any arguments - I need an email address, subject and body\n");
		return -1;
	}
	LOCAL_USER_ADD(u);
	/* Let her cry..... */

        s = ast_strdupa((void *) data);

        emailaddress = strsep(&s, "|");
        temailfrom    = strsep(&s, "|");
        emailsubject = strsep(&s, "|");
        emailbody    = strsep(&s, "|");
	
					
	/* check to see if params passed */

        if (!emailaddress || !emailsubject || !emailbody) {
                ast_log(LOG_WARNING, "SendEmail requires an email address, subject and message body as arguments\n");
                LOCAL_USER_REMOVE(u);
                return -1;
	}	
	
	/* check we have a from address */
	if (!strcmp(temailfrom,"")) 
		res = send_em_message(emailaddress,emailfrom,emailsubject,emailbody,chan->callerid, chan->exten); 
	else	
		res = send_em_message(emailaddress,temailfrom,emailsubject,emailbody,chan->callerid, chan->exten);
	
	LOCAL_USER_REMOVE(u);
	return res;
}


static int load_config(void)
{
	struct ast_config *cfg;		
        char *astemail;
        char *emailcmd     = SENDMAIL;
	char *emailfrom    = DEFAULT_FROM;	
	char *emailsubject = DEFAULT_SUBJECT;
	char *emailbody    = DEFAULT_BODY;

        cfg = ast_load(EMAIL_CONFIG);

        if (cfg) {
                /* General settings */
                /* Mail command */
                strncpy(mailcmd, SENDMAIL, sizeof(mailcmd) - 1); /* Default */
                if ((emailcmd = ast_variable_retrieve(cfg, "general", "mailcmd")))
                	strncpy(mailcmd, emailcmd, sizeof(mailcmd) - 1); 

                /* email from  */
                strncpy(mailfrom, DEFAULT_FROM, sizeof(mailfrom) - 1); /* Default */
                if ((emailfrom = ast_variable_retrieve(cfg, "general", "mailfrom")))
        	        strncpy(mailfrom, emailfrom, sizeof(mailfrom) - 1); 

                /* email subject */
                strncpy(mailsubject, DEFAULT_SUBJECT, sizeof(mailsubject) - 1); /* Default */
                if ((emailsubject = ast_variable_retrieve(cfg, "general", "mailsubject")))
	                strncpy(mailsubject, emailsubject, sizeof(mailsubject) - 1); 

                /* email body */
                strncpy(mailbody, DEFAULT_BODY, sizeof(mailbody) - 1); /* Default */
                if ((emailbody = ast_variable_retrieve(cfg, "general", "mailbody"))) 
			strncpy(mailbody, emailbody, sizeof(mailbody) - 1);

		ast_destroy(cfg);
		return 0;
		
        }                    
	else 
		return -1;

}

static int handle_cli_sendemail_exec(int fd, int argc, char **argv)
{

	char *eto, *efrom, *esubject, *ebody;
	int res;

	if (argc < 4) {
		 ast_cli(fd, "\n\nToo few parameters!\nUseage:\n\temail to from subject body\n");
		 return -1;
	}
	

	if (argc <5) {
		efrom = mailfrom;
                eto = argv[1];
                esubject = argv[2];
                ebody = argv[3];
	}							
	else {
		eto = argv[1];
		efrom = argv[2];
	        esubject = argv[3];
	        ebody = argv[4];
	}

	res = send_em_message(eto,efrom,esubject,ebody,NULL,NULL);
        ast_cli(fd, "Sending message to %s\n",eto);
        return res;
}
		
static char sendemail_usage[] =
"send email to-address, from-name,subject,body.\n";
		
static struct ast_cli_entry cli_sendemail = {
        { "email", NULL, NULL }, handle_cli_sendemail_exec,
	"Send an email message", sendemail_usage, NULL };
		
int unload_module(void)
{
     int res;
     
     STANDARD_HANGUP_LOCALUSERS;
     
     ast_cli_unregister(&cli_sendemail);
     res = ast_unregister_application(app_sendemail);
     res |= ast_unregister_application(app_emailmissedcall);
     
     return res;
}

int load_module(void)
{
     int res;
     
     res = ast_register_application(app_sendemail, sendemail_exec, sendemail_synopsis, sendemail_descrip);
     res |= ast_register_application(app_emailmissedcall, emailmissedcall_exec, emailmissedcall_synopsis, emailmissedcall_descrip);
     ast_cli_register(&cli_sendemail);
     
    
     if (res)
             return(res);

     if ((res=load_config())) {
             return(res);
     }

     return res;
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}



/* You and me, we come from different worlds... */




